local _, Addon = ...
local ActionBar = PetBattleFrame.BottomFrame
local Battle, Journal = C_PetBattles, C_PetJournal
local Actions = Addon:NewModule('EnemyActions', 'CheckButton', nil, ActionBar)

local MODIFIER_ICON = 'Interface\\PetBattles\\BattleBar-AbilityBadge-'
local ENEMY = LE_BATTLE_PET_ENEMY
local ALLY = LE_BATTLE_PET_ALLY


--[[ Startup ]]--

function Actions:Startup()
	self.__index = self
	self:SetPoint('BOTTOM', ActionBar, 'TOP')
	self:SetSize(300, 100)
	self:SetScale(.8)
	
	self:SetHook('PetBattlePetSelectionFrame_Show', self.Hide)
	self:SetHook('PetBattlePetSelectionFrame_Hide', self.Show)
	
	for i = 1, NUM_BATTLE_PET_ABILITIES do
		self:CreateAbility(i)
	end
end

function Actions:SetHook(target, hook)
	hooksecurefunc(target, function()
		hook(self)
	end)
end

function Actions:CreateAbility(i)
	local button = CreateFrame('CheckButton', nil, self, 'PetBattleAbilityButtonTemplate', i)
	setmetatable(button, self)
	
	button:SetPoint('LEFT', (button:GetWidth() + 5) * i, 10)
	button:SetHighlightTexture(nil)
	button:SetPushedTexture(nil)
	
	button:SetScript('OnEvent', self.UpdateAbility)
	button:SetScript('OnShow', self.UpdateAbility)
	button:SetScript('OnEnter', self.OnEnter)
	button:SetScript('OnClick', nil)
	
	button:UnregisterAllEvents()
	button:RegisterEvent('PET_BATTLE_PET_CHANGED')
	button:RegisterEvent('PET_BATTLE_PET_ROUND_PLAYBACK_COMPLETE')
end


--[[ Update ]]--

function Actions:UpdateAbility()
	local id, name, icon, _,_,_, type = self:GetAbilityInfo()
	local modifier, usable, level, cooldown, onCooldown
	
	if name then
		modifier = Battle.GetAttackModifier(type, self:GetPetType(ALLY))
		usable, cooldown = self:GetAbilityState()
		onCooldown = cooldown and cooldown > 0
		usable = usable and not onCooldown
	else  
		id, level = self:FindAbilityID()
		name, icon = Journal.GetPetAbilityInfo(id)
		modifier = 1
	end
	
	self:UpdateIcon(icon, usable, level)
	self:UpdateCooldown(onCooldown, cooldown)
	self:UpdateModifier(modifier)
	self.id, self.level = id, level
end

function Actions:UpdateIcon(icon, usable, locked)
	local color = usable and 1 or .5
	self.Icon:SetTexture(icon)
	self.Icon:SetDesaturated(not usable)
	self.Icon:SetVertexColor(color, color, color)
	self.Lock:SetShown(locked)
end

function Actions:UpdateCooldown(onCooldown, cooldown)
	self.Cooldown:SetText(cooldown or "")
	self.CooldownShadow:SetShown(onCooldown)
	self.Cooldown:SetShown(onCooldown)
end

function Actions:UpdateModifier(modifier)
	self.BetterIcon:SetTexture(MODIFIER_ICON .. (modifier > 1 and 'Strong' or 'Weak'))
	self.BetterIcon:SetShown(modifier ~= 1)
end


--[[ Tooltip ]]--

function Actions:OnEnter()
	local requiredLevel = self.level and PET_ABILITY_REQUIRES_LEVEL:format(self.level)
	local setTip = PetBattleAbilityTooltip_SetAbilityByID

	self:DoForOwner(ENEMY, setTip, self.id, requiredLevel)
	self:ShowTip('BOTTOMRIGHT', UIParent, 'BOTTOMRIGHT', -5, 120)
end

function Actions:ShowTip(...)
	PetBattleAbilityTooltip_Show(...)
end


--[[ Abilities ]]--

function Actions:FindAbilityID()
	local abilities, levels = Journal.GetPetAbilityList(self:GetPetSpecie(ENEMY))
	return abilities[self:GetID()], levels[self:GetID()]
end

function Actions:GetAbilityInfo()
	return self:GetAbilityData('GetAbilityInfo')
end

function Actions:GetAbilityState()
	return self:GetAbilityData('GetAbilityState')
end

function Actions:GetAbilityData(func)
	return self:GetBattleData(ENEMY, func, self:GetID())
end


--[[ Battle ]]--

function Actions:GetPetSpecie(owner)
	return self:GetBattleData(owner, 'GetPetSpeciesID')
end

function Actions:GetPetType(owner)
	return self:GetBattleData(owner, 'GetPetType')
end

function Actions:GetBattleData(owner, func, ...)
	return self:DoForOwner(owner, Battle[func], ...)
end

function Actions:DoForOwner(owner, func, ...)
	local pet = Battle.GetActivePet(owner)
	return func(owner, pet, ...)
end

Actions:Startup()