local ADDON, Addon = ...
local Advantage = Addon:NewModule('Switch', 'Frame', ADDON..'Switch', UIParent, 'ButtonFrameTemplate')

local Battle = C_PetBattles
local Pets = Addon.BattlePets
local Ability = Addon.Ability

local NUM_PETS = NUM_BATTLE_PETS_IN_BATTLE
local NUM_ABILITIES = NUM_BATTLE_PET_ABILITIES
local ENEMY = LE_BATTLE_PET_ENEMY
local ALLY = LE_BATTLE_PET_ALLY


--[[ Startup ]]--

function Advantage:Startup()
	self:RegisterEvent('PET_BATTLE_TURN_STARTED')
	self:SetScript('OnHide', PetBattlePetSelectionFrame_Hide)
	self:SetScript('OnEvent', self.Hide)
	self:Hide()
	
	SetPortraitToTexture(self.portrait:GetName(), 'Interface/Icons/INV_Pet_SwapPet')
	PetBattlePetSelectionFrame_Show = function()
		self:Initialize()
		self:UpdateWindow()
		self:Show()
	end
end

function Advantage:Initialize()
	self.Initialize = function() end
	self:CreateSlotLine(ENEMY, 'Enemy Pets', 'TOPRIGHT', -10)
	self:CreateSlotLine(ALLY, 'Player Pets', 'TOPLEFT', 10)
	self.TitleText:SetText(SWITCH_PET)
	self:SetPoint('CENTER')
	self:SetSize(840, 424)
end

function Advantage:CreateSlotLine(owner, title, ...)
	for i = 1, NUM_PETS do
		self[owner..i] = self:CreateSlot(owner, i, ...)
	end
	
	local border = 	CreateFrame('Frame', '$parentBorder'..owner, self, ADDON..'SlotBorder')
	border:SetPoint('TOP', self[owner..1], 0, 5)
	border.title:SetText(title)
end

function Advantage:CreateSlot(owner, i, point, off)
	local slot = self:NewFrame('Button', '$parentSlot' .. owner..i, self.Inset, ADDON..'Slot')
	slot:SetScript('OnShow', slot.UpdateSlot)
	slot:SetScript('OnClick', slot.OnClick)
	slot:SetPoint(point, off, 98 - 108 * i)
	slot.pet = Pets:Get(owner, i)
	
	return slot
end


--[[ Update ]]--

function Advantage:UpdateWindow()
	local best = self:GetBestAllyFor(1)
	
	for i = 1, NUM_PETS do
		local slot = self[ALLY..i]
		slot.bestGlow:SetShown(i == best)
	end
end

function Advantage:UpdateSlot()
	local pet = self.pet
	if pet:Exists() then
		local name, specieName, icon, typeIcon, model = pet:GetDisplayInfo()
		local health, maxHealth, power, speed, level = pet:GetStats()
		local isDead = not pet:IsAlive()
	
		self.isDead:SetShown(isAlive)
		self.disableFrame:SetShown(isAlive)
	
		self.name:SetText(name)
		self.subName:SetText(specieName)
		self.icon:SetTexture(icon)
		self.type:SetTexture(typeIcon)
		self.model:SetDisplayInfo(model)
	
		self.health.text:SetText(format('%d/%d', health, maxHealth))
		self.health:SetMinMaxValues(0, maxHealth)
		self.health:SetValue(health)
	
		self.power:SetText(power)
		self.speed:SetText(speed)
		self.level:SetText(level)
	
		for i = 1, NUM_ABILITIES do
			Ability.Update(self['ability'..i], pet)
		end
	end
end

function Advantage:OnClick()
	if self.pet:CanSwap() then
		self.pet:Swap()
		Advantage:Hide()
	end
end


--[[ Math ]]--

function Advantage:GetBestAllyFor(enemy)
	local enemy = Pets:Get(ENEMY, enemy)
	local bestAdvantage = 0
	local best

	for i = 1, Pets:GetNum(ALLY) do
		local ally = Pets:Get(ALLY, i)
		if ally:IsAlive() then
			local advantage = self:GetPetAdvantage(ally, enemy)
		
			if advantage > bestAdvantage then
				bestAdvantage = advantage
				best = i
			end
		end
	end
	
	return best
end


--[[ Formula ]]--

function Advantage:GetPetAdvantage(caster, target)
	local abilities = self:GetAbilitiesAdvantage(caster, target) / self:GetAbilitiesAdvantage(target, caster)
	local levels = self:GetLevelAdvantage(caster, target)
	
	return abilities * levels
end

function Advantage:GetAbilitiesAdvantage(caster, target)
	local targetType = target:GetType()
	local result = 1
	
	for i = 1, NUM_ABILITIES do
		local castType, isHealing = caster:GetAbilityType(i)
		if castType and not isHealing then
			result = result * Battle.GetAttackModifier(castType, targetType)
		end
	end
	
	return result
end

function Advantage:GetLevelAdvantage(caster, target)
	return caster:GetLevel() / target:GetLevel() ^ 1.12
end

Advantage:Startup()